@@ -1,11 +1,24 @@
+import hashlib
+import ntpath
 import random
 import string
+import sys
 import time
+import os
 import logging
+
+import six
 import wrapt
 import traceback
 import datetime
+import shutil
 import asyncio.exceptions
+
+from functools import lru_cache
+from tornado.log import LogFormatter as _LogFormatter
+
+from sys import platform
+from shutil import copy2
 from json import JSONEncoder, dumps
 from asyncio import sleep
 from base64 import b64encode
@@ -330,10 +343,100 @@ class JSONEncoderWithBytes(JSONEncoder):
                 else:
                     raise ex
 
+
 def format_error(ex):
     return '[{}] {}\n{}'.format(type(ex), ex, traceback.format_exc())
 
 
+def utf8(string):
+    """
+    Make sure string is utf8 encoded bytes.
+
+    If parameter is a object, object.__str__ will been called before encode as bytes
+    """
+    if isinstance(string, six.text_type):
+        return string.encode('utf8')
+    elif isinstance(string, six.binary_type):
+        return string
+    else:
+        return six.text_type(string).encode('utf8')
+
+
+def get_md5(s):
+    md5 = hashlib.md5(utf8(s)).hexdigest()
+    return str(md5)
+
+
+def get_file_md5(file):
+    md5 = hashlib.md5()
+    f = open(file, 'rb')
+    md5.update(f.read())
+    f.close()
+    return str(md5.hexdigest()).lower()
+
+
+class LogFormatter(_LogFormatter, object):
+    def __init__(self, fmt=None, datefmt=None, color=True, *args, **kwargs):
+        datefmt = "%Y-%m-%d %H:%M:%S"
+        # '[%(threadName)s]' \
+        if fmt == 1:
+            fmt = '%(color)s[%(asctime)s][%(levelname)1.1s]%(end_color)s%(message)s'
+        self.fmt = fmt or '%(color)s[%(asctime)s][%(levelname)1.1s][%(name)s][%(filename)s:%(lineno)d:%(funcName)s()]%(end_color)s%(message)s'
+
+        super(LogFormatter, self).__init__(color=color, fmt=self.fmt, datefmt=datefmt, *args, **kwargs)
+
+    def formatTime(self, record, datefmt=None):
+        ct = self.converter(record.created)
+        if datefmt:
+            s = str(datetime.datetime.now().strftime(datefmt))
+        else:
+            t = time.strftime("%Y-%m-%d %H:%M:%S", ct)
+            s = "%s.%03d" % (t, record.msecs)
+        return s
+
+
+logger_info_where = logging.getLogger('info_where')
+logger_info_where.setLevel(logging.DEBUG)
+
+
+def set_shell_log(log, fmt=None):
+    # create a file handler
+    handler = logging.StreamHandler()
+    handler.setLevel(logging.DEBUG)
+
+    # create a logging format
+    handler.setFormatter(LogFormatter(fmt=fmt))
+
+    # add the handlers to the logger
+    log.addHandler(handler)
+    if log.name != 'root':
+        log.propagate = False
+
+
+def log_info(*args, **kwargs):
+    # 什么函数调用了此函数
+    which_fun_call_this = sys._getframe(1).f_code.co_name  # NOQA
+    # 获取被调用函数在被调用时所处代码行数
+    line = sys._getframe().f_back.f_lineno
+    # 获取被调用函数所在模块文件名
+    file_name = sys._getframe(1).f_code.co_filename
+    logger_info_where.info(f'"{file_name}:{line}" {which_fun_call_this}()：' + ' '.join(
+        [arg.__str__() for arg in args] + [', ', ', '.join(f"{k}={v}" for k, v in kwargs.items())]))
+
+
+def active_log():
+    # set_shell_log(logger)
+    set_shell_log(logging.getLogger())
+    set_shell_log(logger_info_where, 1)
+
+
+def activate_debug_logger(level=logging.DEBUG):
+    """Global logger used when running from command line."""
+    logging.basicConfig(
+        format='(%(levelname)s) %(message)s', level=level
+    )
+
+
 def with_cur_lock():
     @wrapt.decorator
     def wrapper(func, instance, args, kwargs):
@@ -381,5 +484,106 @@ def gen_pass(val_type='all', val_len=8):
 
     return password_str
 
-if __name__ == '__main__':
-    run_task_auto_retry(run_task_auto_retry)
+
+def remove_folder(path):
+    if os.path.exists(path):
+        shutil.rmtree(path)
+
+
+def makedirs(path):
+    folder_name = path.split('\\')[-1].split('/')[-1]
+    is_folder = '.' not in folder_name
+    path = os.path.realpath(path)
+    out_f_path = os.path.dirname(path)
+    if is_folder:
+        out_f_path = os.path.join(out_f_path, folder_name)
+    if not os.path.exists(out_f_path):
+        logger.info(f'正在创建新文件夹：{out_f_path}，因为{path}需要')
+        os.makedirs(out_f_path)
+        
+
+def iter_path(path, exclude=None, include=None, exclude_path=None, include_path=None, return_type=2, open_kwargs=None):
+    if exclude is None:
+        exclude = []
+    if include is None:
+        include = []
+    if exclude_path is None:
+        exclude_path = []
+    if include_path is None:
+        include_path = []
+    if open_kwargs is None:
+        open_kwargs = {}
+    logging.info(f'开始遍历文件夹：{path}')
+    cnt = 0
+    cnt_size = 0
+    cnt_iter = 0
+    for root, fs, fns in os.walk(path):
+        for fn in fns:
+            cnt += 1
+            if include and not any([bool(include_str in fn) for include_str in include]):
+                continue
+            if exclude and any([bool(exclude_str in fn) for exclude_str in exclude]):
+                continue
+            f_path = os.path.join(root, fn)
+            if include_path and not any([bool(include_str in f_path) for include_str in include_path]):
+                continue
+            if exclude_path and any([bool(exclude_str in f_path) for exclude_str in exclude_path]):
+                continue
+            cnt_size += os.path.getsize(f_path)
+            cnt_iter += 1
+            logging.debug(f'遍历到文件：{f_path} [已处理{cnt_iter}|返回{cnt_iter}个：{cnt_size / 1024 / 1024:.3f}MB]')
+            if return_type == 0:
+                yield cnt, root, *os.path.splitext(fn)
+            elif return_type == 1:
+                yield cnt, root, fn
+            elif return_type == 2:
+                yield cnt, f_path
+            elif return_type == 3:
+                yield f_path
+            elif return_type == 4:
+                yield cnt, open(f_path, 'rb', **open_kwargs)
+            elif return_type == 5:
+                yield open(f_path, 'rb', **open_kwargs)
+    logging.info(f'遍历文件结束 [{cnt_size / 1024 / 1024:.3f}MB] [已处理{cnt_iter}|返回{cnt_iter}个]：{path}')
+
+
+def path_without_leaf(path):
+    result = ""
+    try:
+        head, tail = ntpath.split(path)
+        result = head
+    except Exception as inst:
+        logger.exception(f"path_without_leaf Error - {path}: {inst}")
+    return result
+
+
+# return the name of the file + extension
+def path_leaf(path):
+    result = ""
+    try:
+        head, tail = ntpath.split(path)
+        result = tail or ntpath.basename(head)
+    except Exception as inst:
+        logger.exception(f"Path_leaf Error - {path}: {inst}")
+    return result
+
+
+def create_temporary_copy(path, temp_dir, file_name):
+    if platform == "win32":
+        file_to_check = temp_dir + '\\' + file_name
+    else:
+        file_to_check = temp_dir + '/' + file_name
+    # if the file already exists, we will delete it
+    exists = os.path.isfile(file_to_check)
+    if exists:
+        os.chmod(file_to_check, 777)
+        os.remove(file_to_check)
+    temp_path = os.path.join(temp_dir, file_name)
+    copy2(path, temp_path)
+    return temp_path
+
+
+def time_stamp():
+    """returns a formatted current time/date"""
+    import time
+    return str(time.strftime("%Y_%m_%d_%H_%M_%S"))
